"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// ../packages/core/index.ts
var core_exports = {};
__export(core_exports, {
  run: () => run
});
module.exports = __toCommonJS(core_exports);
var process2 = __toESM(require("process"), 1);
var import_utils2 = require("../utils/index.cjs");
var import_cac = require("cac");

// ../package.json
var version = "1.2.0";

// ../packages/core/storage.ts
var import_fs = require("fs");
var import_path = require("path");
var import_os = require("os");
var storage;
var storageDir = (() => {
  const home = (0, import_os.homedir)();
  const oldPath = (0, import_path.resolve)(home, "pr-checker-storage");
  if ((0, import_fs.existsSync)(oldPath))
    return oldPath;
  const dotPath = (0, import_path.resolve)(home, ".config/pr-checker-storage");
  if (!(0, import_fs.existsSync)(dotPath)) {
    (0, import_fs.mkdirSync)(dotPath, {
      recursive: true
    });
  }
  return dotPath;
})();
var storagePath = (0, import_path.resolve)(storageDir, "_pr_checker_storage.json");
async function loadStorage(fn) {
  if (!storage) {
    storage = (0, import_fs.existsSync)(storagePath) ? JSON.parse(await import_fs.promises.readFile(storagePath, "utf-8") || "{}") || {} : {};
  }
  if (fn) {
    if (await fn(storage))
      await saveStorage();
  }
  return storage;
}
async function saveStorage() {
  if (storage) {
    if (!(0, import_fs.existsSync)(storageDir))
      await import_fs.promises.mkdir(storageDir, { recursive: true });
    await import_fs.promises.writeFile(storagePath, JSON.stringify(storage), "utf-8");
  }
}
async function clearStorage() {
  if (storage) {
    if (!(0, import_fs.existsSync)(storageDir))
      await import_fs.promises.mkdir(storageDir, { recursive: true });
    await import_fs.promises.writeFile(storagePath, "{}", "utf-8");
  }
}

// ../packages/core/runtime.ts
var import_prompts = __toESM(require("prompts"), 1);
var import_ora = __toESM(require("ora"), 1);
var import_utils = require("../utils/index.cjs");
var import_console_table_printer = require("console-table-printer");
var import_chalk2 = __toESM(require("chalk"), 1);

// ../packages/core/command-option.ts
var import_chalk = __toESM(require("chalk"), 1);
var typeOption = [{
  type: "select",
  name: "typeSelect",
  message: "Detect all Repo's PR?",
  choices: ["All Repo", "Detect the PR of a certain Repo"].map((item) => {
    let value = "all";
    if (item !== "All Repo")
      value = "select";
    return { title: item, value };
  })
}];
var createRepoOption = (list) => {
  const handler = (item) => {
    return { title: item, value: item };
  };
  return [{
    type: "select",
    name: "RepoSelect",
    message: "Please select a Repo",
    choices: list.map(handler)
  }];
};
var createPrOption = (list) => {
  const handler = (item) => {
    const repo = import_chalk.default.blueBright.bold(`[${item.repo}]`);
    const number = import_chalk.default.yellowBright.bold(`[#${item.number}]`);
    const title = `${item.title}`;
    const canMerge = item.isNeedUpdate ? import_chalk.default.yellowBright.bold("<CanMerge:true>") : import_chalk.default.redBright.bold("<CanMerge:false>");
    const infoTitle = `${canMerge}: ${repo}-${number} -> ${title}`;
    return {
      title: infoTitle,
      value: {
        title: item.title,
        number: item.number,
        repo: item.repo,
        isNeedUpdate: item.isNeedUpdate,
        reason: item.reason,
        infoTitle
      }
    };
  };
  return [{
    type: "multiselect",
    name: "prSelect",
    message: "Please select the PR that needs to be updated",
    optionsPerPage: 20,
    choices: list.map(handler),
    onRender() {
      if (this.firstRender) {
        this.renderDoneOrInstructions = function() {
          return "";
        };
      }
    }
  }];
};

// ../packages/core/gitApi.ts
var import_core = require("@octokit/core");
var GitApi = class {
  octokit;
  owner;
  forkRepoCache;
  constructor(token, owner) {
    this.octokit = new import_core.Octokit({
      auth: token
    });
    this.owner = owner;
    this.forkRepoCache = {};
  }
  /**
   * 获取账户下所有 pr
   * @param username
   */
  async getPRList(username = this.owner) {
    try {
      const { data } = await this.octokit.request("GET /search/issues", {
        q: `is:pr is:open author:${username}`,
        per_page: 1e3
      });
      const res = {};
      data.items.forEach((val) => {
        const repo = val.repository_url.split("repos/")[1];
        if (!res[repo])
          res[repo] = [];
        res[repo].push({
          title: val.title,
          number: val.number,
          repo,
          id: val.id
        });
      });
      return res;
    } catch (error) {
      console.log(error);
      return [];
    }
  }
  /**
   * Get pr details according to pr number and repo name
   * @param pull_number pr 号
   * @param repo_name upstream Repo name
   * @param title
   */
  async getPRByRepo(pull_number, repo_name, title) {
    try {
      const { data } = await this.octokit.request(
        `GET /repos/${repo_name}/pulls/{pull_number}`,
        {
          pull_number
        }
      );
      const forkRepoName = data.head.repo.full_name;
      const forkRepoDefaultBranch = data.head.repo.default_branch;
      if (!this.forkRepoCache[forkRepoName]) {
        const { data: forkRepoData } = await this.octokit.request(
          `GET /repos/${forkRepoName}/branches/{default_branch_name}`,
          {
            default_branch_name: forkRepoDefaultBranch
          }
        );
        this.forkRepoCache[forkRepoName] = {
          sha: forkRepoData.commit.sha
        };
      }
      return {
        sha: this.forkRepoCache[forkRepoName].sha,
        // Fork warehouse default branch hash
        org_repo: data.base.repo.full_name,
        org_repo_default_branch: data.base.repo.default_branch,
        pr_sha: data.head.sha,
        pr_repo: forkRepoName,
        pr_repo_default_branch: forkRepoDefaultBranch,
        merged: data.merged,
        mergeable: data.mergeable,
        mergeable_state: data.mergeable_state,
        number: pull_number,
        repo: repo_name,
        title: title || ""
      };
    } catch (error) {
      console.error(error);
      return {};
    }
  }
  /**
   * Determine whether pr wants to synchronize the upstream Repo
   * @param repo_name upstream Repo name
   * @param pr_info The getPRByRepo function returns the result
   */
  async needUpdate(repo_name, pr_info) {
    try {
      if (pr_info.mergeable_state === "dirty")
        return { isNeedUpdate: false, reason: "code conflict" };
      const { data } = await this.octokit.request(
        `GET /repos/${repo_name}/commits`
      );
      if (data.length >= 0) {
        const cur_sha = data[0].sha;
        if (pr_info.sha !== cur_sha && pr_info.mergeable && !pr_info.merged)
          return { isNeedUpdate: true };
        else
          return { isNeedUpdate: false, reason: "not updated" };
      } else {
        return { isNeedUpdate: false, reason: "unknown error" };
      }
    } catch (error) {
      console.error(error);
      return { isNeedUpdate: false, reason: "unknown error" };
    }
  }
  /**
   * update pr
   * @param pull_number
   * @param repo_name
   */
  async updatePR(pull_number, repo_name) {
    try {
      const res = await this.octokit.request(
        `PUT /repos/${repo_name}/pulls/{pull_number}/update-branch`,
        {
          pull_number
        }
      );
      return res.data.message;
    } catch (error) {
      console.error(error);
      return {};
    }
  }
};
async function getUserName(token) {
  const octokit = new import_core.Octokit({
    auth: token
  });
  const { data } = await octokit.request("GET /user");
  return data;
}

// ../packages/core/runtime.ts
var promptsRun = async (option) => {
  const res = await (0, import_prompts.default)(option, {
    onCancel: () => {
      (0, import_utils.log)("info", "Operation cancel");
      process.exit();
    }
  });
  return { ...res };
};
async function runtimeStart(store) {
  const isAllRepo = await promptsRun(typeOption);
  let updateRes = [];
  const spinner = (0, import_ora.default)({ text: "Loading Repo......", color: "blue" }).start();
  const githubApi = new GitApi(store.token, store.username);
  const prList = await githubApi.getPRList();
  spinner.stop();
  const repoList = Object.keys(prList);
  if (isAllRepo.typeSelect === "select") {
    const selectRepo = await promptsRun(createRepoOption(repoList));
    const prl = prList[selectRepo.RepoSelect];
    (0, import_utils.log)("info", `Checking PR by ${selectRepo.RepoSelect}......`);
    const prListByRepo = await checkPR(prl, githubApi);
    const prSelectRes = await promptsRun(createPrOption(prListByRepo));
    (0, import_utils.log)("info", `Update PR by ${selectRepo.RepoSelect}......`);
    updateRes = await updatePR(prl, prSelectRes, githubApi);
  } else {
    const prl = [];
    repoList.forEach((val) => {
      prList[val].forEach((item) => {
        prl.push(item);
      });
    });
    (0, import_utils.log)("info", "Checking PR......");
    const prListByRepo = await checkPR(prl, githubApi);
    const prSelectRes = await promptsRun(createPrOption(prListByRepo));
    (0, import_utils.log)("info", "Update PR......");
    updateRes = await updatePR(prl, prSelectRes, githubApi);
  }
  spinner.succeed();
  (0, import_utils.log)("success", "\u2714 All PR updates completed");
  await printUpdateRes(updateRes);
}
async function checkPR(prl, githubApi) {
  if (prl.length === 0) {
    (0, import_utils.log)("error", "Please select a pr to check");
    process.exit();
  }
  const prListByRepo = await Promise.all(createRunList(prl.length, async (i) => {
    const prInfo = await githubApi.getPRByRepo(prl[i].number, prl[i].repo, prl[i].title);
    const res = await githubApi.needUpdate(prl[i].repo, prInfo);
    (0, import_utils.log)("success", `\u2714 Check PR #${prl[i].number} completed`);
    return {
      ...prInfo,
      ...res
    };
  }));
  return prListByRepo;
}
async function updatePR(prl, prSelectRes, githubApi) {
  const updateRes = await Promise.all(createRunList(prl.length, async (i) => {
    if (prSelectRes.prSelect[i] && prSelectRes.prSelect[i].isNeedUpdate) {
      await githubApi.updatePR(prSelectRes.prSelect[i].number, prSelectRes.prSelect[i].repo);
      (0, import_utils.log)("success", `\u2714 Update PR #${prl[i].number} completed`);
    }
    return {
      ...prSelectRes.prSelect[i]
    };
  }));
  return updateRes;
}
async function printUpdateRes(res) {
  const p = new import_console_table_printer.Table({
    columns: [
      { name: "number", alignment: "left" },
      { name: "can merge", alignment: "left" },
      { name: "success", alignment: "left" },
      { name: "reason", alignment: "left" },
      { name: "repo", alignment: "left" },
      { name: "title", alignment: "left" }
    ]
  });
  res.sort((a, b) => Number(a.isNeedUpdate) - Number(b.isNeedUpdate));
  res.forEach((item) => {
    if (!(0, import_utils.isEmptyObj)(item)) {
      p.addRow({
        "number": import_chalk2.default.greenBright.bold(`#${item.number}`),
        "can merge": item.isNeedUpdate ? import_chalk2.default.greenBright.bold("true") : import_chalk2.default.redBright.bold("false"),
        "success": item.isNeedUpdate ? import_chalk2.default.greenBright.bold("true") : import_chalk2.default.redBright.bold("false"),
        "reason": item.isNeedUpdate ? import_chalk2.default.blueBright.bold("-") : item.reason === "not updated" ? import_chalk2.default.blueBright.bold(item.reason) : import_chalk2.default.yellowBright.bold(item.reason),
        "repo": import_chalk2.default.blueBright.bold(`<${item.repo}>`),
        "title": item.title
      });
    }
  });
  p.printTable();
}
function createRunList(taskNum, taskFunc) {
  const taskList = [];
  for (let i = 0; i < taskNum; i++) {
    taskList.push(new Promise((resolve2) => {
      resolve2(taskFunc(i));
    }));
  }
  return taskList;
}

// ../packages/core/index.ts
var cli = (0, import_cac.cac)("pr-checker");
var run = async () => {
  const storage2 = await loadStorage();
  cli.option("-t <token>,--token <token>", "set github token");
  cli.option("-u <username>, --username <username>", "set github username");
  cli.option("-c, --clear", "clear token and username");
  cli.option("-g, --get", "get git config");
  cli.command("run", "check your pr").action(async () => {
    if (!storage2.token) {
      (0, import_utils2.log)("error", "use `pr-checker -t <TOKEN>` to set your token");
      process2.exit(1);
    }
    if (!storage2.username) {
      (0, import_utils2.log)("info", "You have not set a username, it has been automatically set for you according to the token");
      const { login } = await getUserName(storage2.token);
      await setUserName(login);
    }
    await runtimeStart(storage2);
  });
  cli.help();
  cli.version(version);
  const parseRes = cli.parse();
  const {
    u,
    username,
    t,
    token,
    c,
    clear,
    h,
    help,
    v,
    version: versions,
    g,
    get
  } = parseRes.options;
  if (h || help || v || versions)
    return;
  if (c || clear) {
    if (storage2) {
      await clearStorage();
      (0, import_utils2.log)("success", "clear token and username successfully");
      return;
    }
  }
  if (g || get) {
    if (storage2) {
      (0, import_utils2.log)("info", `username: ${storage2.username || "null"}`);
      (0, import_utils2.log)("info", `token: ${storage2.token || "null"}`);
      return;
    }
  }
  if (t && typeof t === "string" || token && typeof token === "string") {
    storage2.token = t || token;
    await saveStorage();
    (0, import_utils2.log)("success", "token set successfully");
    return;
  }
  if (u && typeof u === "string" || username && typeof username === "string") {
    await setUserName();
    (0, import_utils2.log)("success", "username set successfully");
  }
  async function setUserName(name) {
    storage2.username = name || u || username;
    await saveStorage();
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  run
});
